package adder

import Chisel._

class Adder4 extends Component {
  val io = new Bundle {
    val a = Bits(INPUT, 4)
    val b = Bits(INPUT, 4)
    val ci = Bits(INPUT, 1)
    val s = Bits(OUTPUT, 4)
    val co = Bits(OUTPUT, 1)
  }

  val fa0 = new FullAdder()
  val fa1 = new FullAdder()
  val fa2 = new FullAdder()
  val fa3 = new FullAdder()

  def fullAdder_connect(fa: FullAdder, a: Bits, b: Bits, c: Bits) = {
    fa.io.a := a
    fa.io.b := b
    fa.io.c := c
  }

  fullAdder_connect(fa0, io.a(0), io.b(0), io.ci)
  fullAdder_connect(fa1, io.a(1), io.b(1), fa0.io.co)
  fullAdder_connect(fa2, io.a(2), io.b(2), fa1.io.co)
  fullAdder_connect(fa3, io.a(3), io.b(3), fa2.io.co)
  io.s := fa3.io.s ## fa2.io.s ## fa1.io.s ## fa0.io.s
  io.co := fa3.io.co
}

class FullAdder extends Component {
  val io = new Bundle {
    val a = Bits(INPUT, 1)
    val b = Bits(INPUT, 1)
    val c = Bits(INPUT, 1)
    val s = Bits(OUTPUT, 1)
    val co = Bits(OUTPUT, 1)
  }

  val ha0 = new HalfAdder()
  val ha1 = new HalfAdder()
  ha0.io.a := io.a
  ha0.io.b := io.b
  ha1.io.a := ha0.io.s
  ha1.io.b := io.c
  io.s := ha1.io.s
  io.co := ha0.io.co | ha1.io.co
}

class HalfAdder extends Component {
  val io = new Bundle {
    val a = Bits(INPUT, 1)
    val b = Bits(INPUT, 1)
    val s = Bits(OUTPUT, 1)
    val co = Bits(OUTPUT, 1)
  }

  val tmp1 = io.a | io.b
  val tmp2 = io.a & io.b
  io.s := tmp1 & ~tmp2
  io.co := tmp2
}

object adder_main {
  def main(args: Array[String]): Unit = {
    val top = args(0)
    val chiselArgs = args.tail
    chiselMain(chiselArgs.toArray, () => Class.forName(top).newInstance.asInstanceOf[Component])
  }
}
